@@ -465,11 +465,9 @@ class TestCallable(MakeTubsMixin, unittest.TestCase):
             self.failUnlessEqual(len(failures), 1)
             f = failures[0]
             self.failUnless(isinstance(f, failure.Failure))
-            self.failUnless(isinstance(f, call.CopiedFailure))
-            self.failUnless("Traceback (most recent call last):\n"
-                            in str(f))
-            self.failUnless("\nexceptions.ValueError: you asked me to fail\n"
-                            in str(f))
+            self.failUnlessIn("Traceback:", str(f))
+            self.failUnlessIn("exceptions.ValueError", str(f))
+            self.failUnlessIn(": you asked me to fail\n", str(f))
         d.addBoth(_check)
         return d
     testLogLocalFailure.timeout = 2
@@ -545,6 +543,32 @@ class TestCallable(MakeTubsMixin, unittest.TestCase):
         return d
 
 
+
+class TestNotifyOnConnectionLost(unittest.TestCase):
+    """
+    Tests for L{Broker._notifyOnConnectionLost}.
+    """
+    def testCalled(self):
+        """
+        The object passed to L{Broker._notifyOnConnectionLost} is called when the
+        L{Broker} is notify that its connection has been lost.
+        """
+        transport = NullTransport()
+        protocol = broker.Broker(None)
+        protocol.makeConnection(transport)
+        disconnected = []
+        protocol._notifyOnConnectionLost(lambda: disconnected.append(1))
+        protocol._notifyOnConnectionLost(lambda: disconnected.append(2))
+        protocol.connectionLost(failure.Failure(Exception("Connection lost")))
+
+        d = flushEventualQueue()
+        def flushed(ignored):
+            self.assertEqual([1, 2], disconnected)
+        d.addCallback(flushed)
+        return d
+
+
+
 class TestService(unittest.TestCase):
     def setUp(self):
         self.services = [Tub()]
